<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <title>PyBankID QR Demo</title>
  <link crossorigin="anonymous" href="https://stackpath.bootstrapcdn.com/bootstrap/4.3.1/css/bootstrap.min.css"
        integrity="sha384-ggOyR0iXCbMQv3Xipma34MD+dH/1fQ784/j6cY/iJTQUOhcWr7x9JvoRxT2MZw1T" rel="stylesheet">
</head>
<body>
<div class="row d-flex flex-column align-items-center">
  <div class="col-6 text-center pt-5">
    <a href="/"><h1>PyBankID QR Code Demo</h1></a>
    <h2>Initiate QR signing</h2>
  </div>
  <div class="col-6 text-left pt-5">
    <form action="/send-initiate" method="post">
      <div class="form-group">
        <label for="personnummer">Personnummer</label>
        <input class="form-control" id="personnummer" name="personnummer" aria-describedby="personnummer-help" placeholder="YYYYMMDDXXXX" type="text">
        <small id="personnummer-help" class="form-text text-muted">Skicka in ett tomt fält för att starta autentisering utan personnummer.</small>
      </div>
      <button class="btn btn-primary" type="submit">Skicka</button>
    </form>
  </div>
</div>

</body>

<script crossorigin="anonymous"
        integrity="sha384-q8i/X+965DzO0rT7abK41JStQIAqVgRVzpbzo5smXKp4YfRvH+8abtTE1Pi6jizo"
        src="https://code.jquery.com/jquery-3.3.1.slim.min.js"></script>
<script crossorigin="anonymous"
        integrity="sha384-UO2eT0CpHqdSJQ6hJty5KVphtPhzWj9WO1clHTMGa3JDZwrnQq4sF86dIHNDz0W1"
        src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.7/umd/popper.min.js"></script>
<script crossorigin="anonymous"
        integrity="sha384-JjSmVgyd0p3pXB1rRibZUAYoIIy6OrQ6VrjIEaFf/nJGzIxFDsf4x0xIM+B07jRM"
        src="https://stackpath.bootstrapcdn.com/bootstrap/4.3.1/js/bootstrap.min.js"></script>

</html>